import { Route } from '@modules/geography/core/domain/route.types';
import { Role } from './ad.types';
import { Waypoint } from './value-objects/waypoint.value-object';

export class PathCreator {
  constructor(
    private readonly roles: Role[],
    private readonly waypoints: Waypoint[],
  ) {}

  public getBasePaths = (): Path[] => {
    const paths: Path[] = [];
    if (
      this.roles.includes(Role.DRIVER) &&
      this.roles.includes(Role.PASSENGER)
    ) {
      if (this.waypoints.length == 2) {
        // 2 points => same route for driver and passenger
        paths.push(this._createGenericPath());
      } else {
        paths.push(this._createDriverPath(), this._createPassengerPath());
      }
    } else if (this.roles.includes(Role.DRIVER)) {
      paths.push(this._createDriverPath());
    } else if (this.roles.includes(Role.PASSENGER)) {
      paths.push(this._createPassengerPath());
    }
    return paths;
  };

  private _createGenericPath = (): Path =>
    this._createPath(this.waypoints, PathType.GENERIC);

  private _createDriverPath = (): Path =>
    this._createPath(this.waypoints, PathType.DRIVER);

  private _createPassengerPath = (): Path =>
    this._createPath(
      [this._firstWaypoint(), this._lastWaypoint()],
      PathType.PASSENGER,
    );

  private _firstWaypoint = (): Waypoint =>
    this.waypoints.find(
      (waypoint: Waypoint) => waypoint.position == 0,
    ) as Waypoint;

  private _lastWaypoint = (): Waypoint =>
    this.waypoints.find(
      (waypoint: Waypoint) =>
        waypoint.position ==
        Math.max(
          ...this.waypoints.map((waypoint: Waypoint) => waypoint.position),
        ),
    ) as Waypoint;

  private _createPath = (waypoints: Waypoint[], type: PathType): Path => ({
    type,
    waypoints,
  });
}

export type Path = {
  type: PathType;
  waypoints: Waypoint[];
};

export type TypedRoute = {
  type: PathType;
  route: Route;
};

export enum PathType {
  GENERIC = 'generic',
  DRIVER = 'driver',
  PASSENGER = 'passenger',
}
