import {
  ArgumentOutOfRangeException,
  ValueObject,
} from '@mobicoop/ddd-library';
import { WaypointProps } from './waypoint.value-object';
import { Actor } from './actor.value-object';
import { Role } from '../ad.types';

/** Note:
 * Value Objects with multiple properties can contain
 * other Value Objects inside if needed.
 * */

export interface WayStepProps extends WaypointProps {
  actors: Actor[];
}

export class WayStep extends ValueObject<WayStepProps> {
  get position(): number {
    return this.props.position;
  }

  get lon(): number {
    return this.props.lon;
  }

  get lat(): number {
    return this.props.lat;
  }

  get actors(): Actor[] {
    return this.props.actors;
  }

  protected validate(props: WayStepProps): void {
    if (props.actors.length <= 0)
      throw new ArgumentOutOfRangeException('at least one actor is required');
    if (
      props.actors.filter((actor: Actor) => actor.role == Role.DRIVER).length >
      1
    )
      throw new ArgumentOutOfRangeException(
        'a waystep can contain only one driver',
      );
  }
}
