import { Controller, UsePipes } from '@nestjs/common';
import { GrpcMethod, RpcException } from '@nestjs/microservices';
import { ResponseBase, RpcValidationPipe } from '@mobicoop/ddd-library';
import { RpcExceptionCode } from '@mobicoop/ddd-library';
import { MatchPaginatedResponseDto } from '../dtos/match.paginated.response.dto';
import { QueryBus } from '@nestjs/cqrs';
import { MatchRequestDto } from './dtos/match.request.dto';
import { MatchQuery } from '@modules/ad/core/application/queries/match/match.query';
import { MatchEntity } from '@modules/ad/core/domain/match.entity';

@UsePipes(
  new RpcValidationPipe({
    whitelist: false,
    forbidUnknownValues: false,
  }),
)
@Controller()
export class MatchGrpcController {
  constructor(private readonly queryBus: QueryBus) {}

  @GrpcMethod('MatcherService', 'Match')
  async match(data: MatchRequestDto): Promise<MatchPaginatedResponseDto> {
    try {
      const matches: MatchEntity[] = await this.queryBus.execute(
        new MatchQuery(data),
      );
      return new MatchPaginatedResponseDto({
        data: matches.map((match: MatchEntity) => ({
          ...new ResponseBase(match),
          adId: match.getProps().adId,
        })),
        page: 1,
        perPage: 5,
        total: matches.length,
      });
    } catch (e) {
      throw new RpcException({
        code: RpcExceptionCode.UNKNOWN,
        message: e.message,
      });
    }
  }
}
