import { PassengerOrientedWaypointsCompleter } from '@modules/ad/core/application/queries/match/completer/passenger-oriented-waypoints.completer';
import { MatchQuery } from '@modules/ad/core/application/queries/match/match.query';
import { AlgorithmType } from '@modules/ad/core/application/types/algorithm.types';
import { Waypoint } from '@modules/ad/core/application/types/waypoint.type';
import { Frequency, Role } from '@modules/ad/core/domain/ad.types';
import { CandidateEntity } from '@modules/ad/core/domain/candidate.entity';

const originWaypoint: Waypoint = {
  position: 0,
  lat: 48.689445,
  lon: 6.17651,
  houseNumber: '5',
  street: 'Avenue Foch',
  locality: 'Nancy',
  postalCode: '54000',
  country: 'France',
};
const destinationWaypoint: Waypoint = {
  position: 1,
  lat: 48.8566,
  lon: 2.3522,
  locality: 'Paris',
  postalCode: '75000',
  country: 'France',
};

const matchQuery = new MatchQuery({
  algorithmType: AlgorithmType.PASSENGER_ORIENTED,
  driver: true,
  passenger: true,
  frequency: Frequency.PUNCTUAL,
  fromDate: '2023-08-28',
  toDate: '2023-08-28',
  schedule: [
    {
      time: '07:05',
    },
  ],
  strict: false,
  waypoints: [originWaypoint, destinationWaypoint],
});

const candidates: CandidateEntity[] = [
  CandidateEntity.create({
    id: 'cc260669-1c6d-441f-80a5-19cd59afb777',
    role: Role.DRIVER,
    waypoints: [
      {
        position: 0,
        lat: 48.678454,
        lon: 6.189745,
      },
      {
        position: 1,
        lat: 48.84877,
        lon: 2.398457,
      },
    ],
  }),
  CandidateEntity.create({
    id: '5600ccfb-ab69-4d03-aa30-0fbe84fcedc0',
    role: Role.PASSENGER,
    waypoints: [
      {
        position: 0,
        lat: 48.689445,
        lon: 6.17651,
      },
      {
        position: 1,
        lat: 48.8566,
        lon: 2.3522,
      },
    ],
  }),
];

describe('Passenger oriented waypoints completer', () => {
  it('should complete candidates', async () => {
    const passengerOrientedWaypointsCompleter: PassengerOrientedWaypointsCompleter =
      new PassengerOrientedWaypointsCompleter(matchQuery);
    const completedCandidates: CandidateEntity[] =
      await passengerOrientedWaypointsCompleter.complete(candidates);
    expect(completedCandidates.length).toBe(2);
  });
});
