import { AD_GET_BASIC_ROUTE_CONTROLLER } from '@modules/ad/ad.di-tokens';
import { Point } from '@modules/ad/core/application/types/point.type';
import { RouteProvider } from '@modules/ad/infrastructure/route-provider';
import { GetBasicRouteControllerPort } from '@modules/geography/core/application/ports/get-basic-route-controller.port';
import { Route } from '@modules/geography/core/domain/route.types';
import { Test, TestingModule } from '@nestjs/testing';

const originPoint: Point = {
  lat: 48.689445,
  lon: 6.17651,
};
const destinationPoint: Point = {
  lat: 48.8566,
  lon: 2.3522,
};

const mockGetBasicRouteController: GetBasicRouteControllerPort = {
  get: jest.fn().mockImplementationOnce(() => ({
    distance: 350101,
    duration: 14422,
    fwdAzimuth: 273,
    backAzimuth: 93,
    distanceAzimuth: 336544,
    points: [
      {
        lon: 6.1765102,
        lat: 48.689445,
      },
      {
        lon: 4.984578,
        lat: 48.725687,
      },
      {
        lon: 2.3522,
        lat: 48.8566,
      },
    ],
  })),
};

describe('Route provider', () => {
  let routeProvider: RouteProvider;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        RouteProvider,
        {
          provide: AD_GET_BASIC_ROUTE_CONTROLLER,
          useValue: mockGetBasicRouteController,
        },
      ],
    }).compile();

    routeProvider = module.get<RouteProvider>(RouteProvider);
  });

  it('should be defined', () => {
    expect(routeProvider).toBeDefined();
  });

  it('should provide a route', async () => {
    const route: Route = await routeProvider.getBasic([
      {
        position: 0,
        ...originPoint,
      },
      {
        position: 1,
        ...destinationPoint,
      },
    ]);
    expect(route.distance).toBe(350101);
    expect(route.duration).toBe(14422);
  });
});
