import {
  ArgumentInvalidException,
  ArgumentOutOfRangeException,
} from '@mobicoop/ddd-library';
import { Step } from '@modules/geography/core/domain/value-objects/step.value-object';

describe('Step value object', () => {
  it('should create a step value object', () => {
    const stepVO = new Step({
      lat: 48.689445,
      lon: 6.17651,
      duration: 150,
      distance: 12000,
    });
    expect(stepVO.duration).toBe(150);
    expect(stepVO.distance).toBe(12000);
    expect(stepVO.lat).toBe(48.689445);
    expect(stepVO.lon).toBe(6.17651);
  });
  it('should throw an exception if longitude is invalid', () => {
    try {
      new Step({
        lat: 48.689445,
        lon: 186.17651,
        duration: 150,
        distance: 12000,
      });
    } catch (e: any) {
      expect(e).toBeInstanceOf(ArgumentOutOfRangeException);
    }
    try {
      new Step({
        lon: 48.689445,
        lat: -186.17651,
        duration: 150,
        distance: 12000,
      });
    } catch (e: any) {
      expect(e).toBeInstanceOf(ArgumentOutOfRangeException);
    }
  });
  it('should throw an exception if latitude is invalid', () => {
    try {
      new Step({
        lat: 248.689445,
        lon: 6.17651,
        duration: 150,
        distance: 12000,
      });
    } catch (e: any) {
      expect(e).toBeInstanceOf(ArgumentOutOfRangeException);
    }
    try {
      new Step({
        lon: -148.689445,
        lat: 6.17651,
        duration: 150,
        distance: 12000,
      });
    } catch (e: any) {
      expect(e).toBeInstanceOf(ArgumentOutOfRangeException);
    }
  });
  it('should throw an exception if distance is invalid', () => {
    try {
      new Step({
        lat: 48.689445,
        lon: 6.17651,
        duration: 150,
        distance: -12000,
      });
    } catch (e: any) {
      expect(e).toBeInstanceOf(ArgumentInvalidException);
    }
  });
  it('should throw an exception if duration is invalid', () => {
    try {
      new Step({
        lat: 48.689445,
        lon: 6.17651,
        duration: -150,
        distance: 12000,
      });
    } catch (e: any) {
      expect(e).toBeInstanceOf(ArgumentInvalidException);
    }
  });
});
